;; Generated package description from eev.el  -*- no-byte-compile: t -*-
(define-package "eev" "20240925" "Support for e-scripts (eepitch blocks, elisp hyperlinks, etc)" '((emacs "25.1")) :commit "43131f4af5f5a01b86731049202d2dd31e0ba25f" :authors '(("Eduardo Ochs" . "eduardoochs@gmail.com")) :maintainer '("Eduardo Ochs" . "eduardoochs@gmail.com") :keywords '("lisp" "e-scripts") :url "http://anggtwu.net/#eev")
